package com.bergerkiller.bukkit.nolagg;

import org.bukkit.Chunk;
import org.bukkit.event.world.ChunkLoadEvent;
import org.bukkit.event.world.ChunkPopulateEvent;
import org.bukkit.event.world.ChunkUnloadEvent;
import org.bukkit.event.world.WorldListener;
import org.bukkit.event.world.WorldLoadEvent;
import org.bukkit.event.world.WorldUnloadEvent;

import com.bergerkiller.bukkit.nolagg.ChunkOperation.Type;
import com.bergerkiller.bukkit.nolagg.meta.ChunkMetaData;
import com.bergerkiller.bukkit.nolagg.meta.WorldMetaData;

public class NLWorldListener extends WorldListener {

	public static Chunk nextIgnoreChunk = null;
	public static long loadedChunks;
	public static long unloadedChunks;
	public static long genChunks;
	
	@Override
	public void onChunkLoad(ChunkLoadEvent event) {
		synchronized (Util.lock) {
			ChunkMetaData data = ChunkMetaData.get(event.getChunk());
			data.touch();
			data.update();
			loadedChunks++;
		}
	}
	
	@Override
	public void onChunkUnload(ChunkUnloadEvent event) {
		synchronized (Util.lock) {
			if (event.getChunk() != nextIgnoreChunk) {
				ChunkMetaData data = ChunkMetaData.get(event.getChunk());
				if (data.isUnloadable() && data.isExpired() && !data.isPlayerNear()) {
					data.remove();
				} else {
					event.setCancelled(true);
					data.touch();
					data.update();
				}
			}
			if (!event.isCancelled()) unloadedChunks++;
		}
	}
		
	@Override
	public void onWorldLoad(WorldLoadEvent event) {
		synchronized (Util.lock) {
			WorldMetaData.get(event.getWorld()).update();
			AutoSaveChanger.change(event.getWorld());
		}
	}
	
	@Override
	public void onWorldUnload(WorldUnloadEvent event) {
		synchronized (Util.lock) {
			WorldMetaData.remove(event.getWorld());
		}
	}
	
	@Override
	public void onChunkPopulate(ChunkPopulateEvent event) {
		if (NoLagg.autoFixLighting) ChunkScheduler.schedule(event, Type.LIGHTING);
		genChunks++;
	}

	
}
